@@ -25,14 +25,13 @@ import six
 from osc_lib.api import auth
 from osc_lib import exceptions
 from osc_lib import session as osc_session
+from osc_lib import version
 
 
 LOG = logging.getLogger(__name__)
 
 PLUGIN_MODULES = []
 
-USER_AGENT = 'osc-lib'
-
 
 class ClientCache(object):
     """Descriptor class for caching created client handles."""
@@ -62,6 +61,8 @@ class ClientManager(object):
         cli_options=None,
         api_version=None,
         pw_func=None,
+        app_name=None,
+        app_version=None,
     ):
         """Set up a ClientManager
 
@@ -73,11 +74,17 @@ class ClientManager(object):
             Callback function for asking the user for a password.  The function
             takes an optional string for the prompt ('Password: ' on None) and
             returns a string containing the password
+        :param app_name:
+            The name of the application for passing through to the useragent
+        :param app_version:
+            The version of the application for passing through to the useragent
         """
 
         self._cli_options = cli_options
         self._api_version = api_version
         self._pw_callback = pw_func
+        self._app_name = app_name
+        self._app_version = app_version
         self.region_name = self._cli_options.region_name
         self.interface = self._cli_options.interface
 
@@ -93,6 +100,8 @@ class ClientManager(object):
         # NOTE(dtroyer): Per bug https://bugs.launchpad.net/bugs/1447784
         #                --insecure overrides any --os-cacert setting
 
+        # Set a hard default
+        self.verify = True
         if self._cli_options.insecure:
             # Handle --insecure
             self.verify = False
@@ -105,7 +114,8 @@ class ClientManager(object):
                 self.cacert = self._cli_options.cacert
             else:
                 # Fall through also gets --verify
-                self.verify = True
+                if self._cli_options.verify is not None:
+                    self.verify = self._cli_options.verify
                 self.cacert = None
 
         # Set up client certificate and key
@@ -117,6 +127,22 @@ class ClientManager(object):
         if self.cert and self._cli_options.key:
             self.cert = self.cert, self._cli_options.key
 
+        # TODO(mordred) The logic above to set all of these is duplicated in
+        # os-client-config but needs more effort to tease apart and ensure that
+        # values are being passed in. For now, let osc_lib do it and just set
+        # the values in occ.config
+        self._cli_options.config['verify'] = self.verify
+        self._cli_options.config['cacert'] = self.cacert
+        # Attack of the killer passthrough values
+        self._cli_options.config['cert'] = self._cli_options.cert
+        self._cli_options.config['key'] = self._cli_options.key
+
+        # TODO(mordred) We also don't have any support for setting or passing
+        # in api_timeout, which is set in occ defaults but we skip occ defaults
+        # so set it here by hand and later we should potentially expose this
+        # directly to osc
+        self._cli_options.config['api_timeout'] = None
+
         # Get logging from root logger
         root_logger = logging.getLogger('')
         LOG.setLevel(root_logger.getEffectiveLevel())
@@ -133,8 +159,6 @@ class ClientManager(object):
     def setup_auth(self):
         """Set up authentication
 
-        :param required_scope: indicate whether a scoped token is required
-
         This is deferred until authentication is actually attempted because
         it gets in the way of things that do not require auth.
         """
@@ -161,13 +185,25 @@ class ClientManager(object):
         LOG.debug('Using parameters %s',
                   strutils.mask_password(self._cli_options.auth))
         self.auth = self._cli_options.get_auth()
+
+        if self._cli_options.service_provider:
+            self.auth = auth.get_keystone2keystone_auth(
+                self.auth,
+                self._cli_options.service_provider,
+                self._cli_options.remote_project_id,
+                self._cli_options.remote_project_name,
+                self._cli_options.remote_project_domain_id,
+                self._cli_options.remote_project_domain_name
+            )
+
         self.session = osc_session.TimingSession(
             auth=self.auth,
             verify=self.verify,
             cert=self.cert,
-            user_agent=USER_AGENT,
+            app_name=self._app_name,
+            app_version=self._app_version,
+            additional_user_agent=[('osc-lib', version.version_string)],
         )
-
         self._auth_setup_completed = True
 
     def validate_scope(self):
@@ -189,11 +225,8 @@ class ClientManager(object):
     @property
     def auth_ref(self):
         """Dereference will trigger an auth if it hasn't already"""
-        if not self._auth_ref:
-            self.setup_auth()
-            LOG.debug("Get auth_ref")
-            self._auth_ref = self.auth.get_auth_ref(self.session)
-        return self._auth_ref
+        self.setup_auth()
+        return self.auth.get_access(self.session)
 
     def is_service_available(self, service_type):
         """Check if a service type is in the current Service Catalog"""
